package api

import (
	"bufio"
	"encoding/json"
	"fmt"
	"io"
	"launcher/conf"
	"launcher/mproto/entity"
	"launcher/socket"
	"os"
	"os/exec"
	"path"
	"path/filepath"
	"regexp"
	"sort"
	"strings"
	"time"

	"github.com/energye/energy/v2/cef"
	"github.com/energye/energy/v2/cef/ipc"
	"github.com/energye/golcl/lcl"
	"github.com/energye/golcl/lcl/types"
)

var Version = "1.0.0"

var NatsPort int

type Config struct {
	Version string
	ExePath string
}

func RegApi(event *cef.BrowserEvent, window cef.IBrowserWindow, bw *cef.LCLBrowserWindow) {

	RegSubWindows(event, window, bw)
	RegSocket(event, window, bw)
	RegUpload(event, window, bw)
	RegZip(event, window, bw)
	RegRunApp(event, window, bw)
	RegUpgrade(event, window, bw)
	// RegShortcut(event, window, bw)

	// 更新配置文件
	// 1.0.0 data/app.json
	ipc.On("UpdateConfig", func(version string) bool {
		fmt.Println("==================更新配置文件信息=========================")
		fmt.Println(version)
		data, err := json.MarshalIndent(&Config{Version: version, ExePath: "data/CR.exe"}, "", "  ")
		if err != nil {
			fmt.Println(err)
			return false
		}
		err = os.WriteFile("data/app.json", data, 0777)
		if err != nil {
			fmt.Println(err)
			return false
		}

		return true
	})

	ipc.On("RunApp", func() bool {
		conf, err := os.Open("data/app.json")
		if err != nil {
			fmt.Println("打开配置文件错误")
			fmt.Println(err)
			return false
		}
		reader := bufio.NewReader(conf)
		decoder := json.NewDecoder(reader)
		config := &Config{}
		err = decoder.Decode(config)
		if err != nil {
			fmt.Println("读取配置文件错误")
			fmt.Println(err)
			return false
		}

		cmd := exec.Command(config.ExePath)
		fmt.Println("程序正在启动...")
		data, err := cmd.Output()
		if err != nil {
			fmt.Println("启动失败")
			fmt.Println(err)
			return false
		}
		fmt.Println(string(data))
		return true
	})

	ipc.On("SelectDir", func(sid int, title string) bool {
		if len(title) < 1 {
			title = "选择目录"
		}
		cbName := fmt.Sprintf("c%d", sid)
		window.RunOnMainThread(func() {
			if ok, dir := lcl.SelectDirectory2(title, "c:/", false); ok {
				dir = strings.ReplaceAll(dir, "\\", "/")
				fmt.Println("选择的目录为：", dir)
				ipc.Emit(cbName, dir)
				return
			}
			ipc.Emit(cbName, "")
		})
		return true
	})

	ipc.On("IsDirEmpty", func(dir string) bool {
		f, err := os.Open(dir)
		if err != nil {
			return false
		}
		defer f.Close()
		_, err = f.Readdir(1)
		return err == io.EOF
	})

	ipc.On("IsFileExit", func(fpath string) bool {
		_, err := os.Stat(fpath)
		if err == nil {
			return true
		}
		if os.IsNotExist(err) {
			return false
		}
		return false
	})

	ipc.On("WriteFileText", func(fpath string, text string) bool {
		dir := path.Dir(fpath)
		os.MkdirAll(dir, os.ModePerm)

		err := os.WriteFile(fpath, []byte(text), os.ModePerm)
		fmt.Println("wreite file error=>", err)

		return err == nil
	})

	ipc.On("ReadFileText", func(dir string) map[string]string {
		out := map[string]string{}
		d, e := os.ReadFile(dir)
		if e != nil {
			fmt.Println(e.Error())
			out["error"] = "文件" + dir + "读取失败!"
			return out
		}
		out["text"] = string(d)
		return out
	})

	ipc.On("CopyFile", func(sid string, src string, target string) bool {
		window.RunOnMainThread(func() {
			CopyFile(src, target)
			ipc.Emit(sid, true)
		})
		return true
	})

	ipc.On("SaveAppConfigItem", func(key string, value string) bool {
		dir := filepath.Join(conf.AppConfig.DataDir, "save")
		os.MkdirAll(dir, os.ModePerm)
		err := os.WriteFile(filepath.Join(dir, key), []byte(value), os.ModePerm)
		if err != nil {
			fmt.Println("[SaveAppConfigItem]", err.Error())
			return false
		}
		return true
	})

	ipc.On("GetAppConfigItem", func(key string) map[string]string {
		out := map[string]string{}
		d, err := os.ReadFile(filepath.Join(conf.AppConfig.DataDir, "save", key))
		if err != nil {
			fmt.Println("[GetAppConfigItem]", err.Error())
			out["error"] = err.Error()
			return out
		}
		out["text"] = string(d)
		return out
	})

	ipc.On("RemoveSysConfigItem", func(key string) bool {
		return os.Remove(filepath.Join(conf.AppConfig.DataDir, "save", key)) == nil
	})

	ipc.On("GetAppDataBaseUrl", func() string {
		return fmt.Sprintf("http://localhost:%d/", conf.AppConfig.AppDataPort)
	})

	ipc.On("CopyFileToAppData", func(src, target string) string {
		ok := CopyFile(src, filepath.Join(conf.AppConfig.DataDir, "appdata", target))
		if !ok {
			return ""
		}
		return fmt.Sprintf("http://localhost:%d/%s", conf.AppConfig.AppDataPort, target)
	})

	ipc.On("RemoveAppData", func(key string) bool {
		return os.Remove(filepath.Join(conf.AppConfig.DataDir, "appdata", key)) == nil
	})

	ipc.On("OpenDir", func(key string) bool {
		if !IsDir(key) {
			key = filepath.Dir(key)
		}
		cmd := exec.Command("cmd", "/c", "start", key)
		er := cmd.Run()
		fmt.Println(er)
		return er == nil
	})

	ipc.On("SelectOneFilePath", func(sid int, title, filters string) bool {

		dlgOpen := lcl.NewOpenDialog(bw)
		dlgOpen.SetFilter(filters)
		//    dlgOpen.SetInitialDir()
		//	dlgOpen.SetFilterIndex()
		dlgOpen.SetOptions(dlgOpen.Options().Include(types.OfShowHelp)) //types.OfAllowMultiSelect  //rtl.Include(, types.OfShowHelp))
		dlgOpen.SetTitle(title)

		cbName := fmt.Sprintf("c%d", sid)
		window.RunOnMainThread(func() {
			if dlgOpen.Execute() {
				fmt.Println("filename: ", dlgOpen.FileName())
				ipc.Emit(cbName, dlgOpen.FileName())
				return
			}
			ipc.Emit(cbName, "")
		})
		return true
	})

	ipc.On("StartProjectHttpServer", func(prjFile string) string {
		if !IsDir(prjFile) {
			prjFile = filepath.Dir(prjFile)
		}
		return startProjectHttp(prjFile)
	})

	var downingMap map[string]bool = make(map[string]bool)
	ipc.On("DownloadFile", func(sid string, fpath string, url string, timeout int) string {
		if downingMap[fpath] {
			return "已经在下载中..."
		}
		downingMap[fpath] = true

		if timeout == 0 {
			timeout = 60 * 20
		}
		go func() {
			eventName := sid
			DownFile(url, fpath, time.Duration(timeout)*time.Second, func(w *WriteCounter) {
				fmt.Println("on start=>")
				socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "start"})

			}, func(w *WriteCounter) {
				socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "progress", Fva11: float32(w.Curr) / float32(w.Total)})
			}, func(err string) {
				fmt.Println("on error=>", err)
				delete(downingMap, fpath)
				socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "error", Error: err})
			}, func() {
				fmt.Println("on succ=>")
				delete(downingMap, fpath)
				socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "succ"})
			})
		}()
		return ""
	})

	ipc.On("SaveFile", func(fpath string, buff []byte) string {
		dir := filepath.Dir(fpath)
		os.MkdirAll(dir, os.ModePerm)
		err := os.WriteFile(fpath, buff, os.ModePerm)
		if err != nil {
			return err.Error()
		}
		fmt.Println("saving over")
		return ""
	})

	ipc.On("GetDirSize", func(dir string) int64 {
		return GetDirSize(dir)
	})

	ipc.On("GetAppDataDir", func() string {
		return strings.ReplaceAll(conf.AppConfig.DataDir, "\\", "/")
	})

	ipc.On("RemoveFile", func(fpath string) bool {
		return os.Remove(fpath) == nil
	})

	ipc.On("RemoveDir", func(fpath string) bool {
		return os.RemoveAll(fpath) == nil
	})

	ipc.On("SetMainTitle", func(title string) bool {
		window.SetTitle(title)
		return true
	})

	//获取nats profile
	ipc.On("NatsProfile", func() map[string]string {
		config := conf.AppConfig
		return map[string]string{
			"apiPort": fmt.Sprintf("%d", config.NatsApiPort),
			"wsPort":  fmt.Sprintf("%d", config.NatsWsPort),
			"ip":      config.Ip,
		}
	})

	ipc.On("GetFilesInDir", func(dir string, filter string) []OutFile {
		out := []OutFile{}
		// 遍历目录及其子目录下的所有文件
		err := filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
			if err != nil {
				fmt.Printf("访问文件或目录时出错：%v\n", err)
				return nil
			}
			// 只统计文件大小，忽略目录
			if !info.IsDir() {
				fpath := strings.ReplaceAll(path, "\\", "/")
				match, err := regexp.MatchString(filter, fpath)
				if err != nil {
					fmt.Println("正则表达式匹配错误:", err)
					return nil
				}
				if match {
					out = append(out, OutFile{Fpath: fpath, CreateAt: info.ModTime().Unix(), Size: info.Size()})
				}
			}
			return nil
		})
		if err != nil {
			fmt.Printf("遍历目录时出错：%v\n", err)
		}
		sort.Sort(OrderedOutFile(out))

		return out
	})

	ipc.On("GetSubDirNames", func(dir string, filter string) []string {

		out := []string{}
		if !IsExist(dir) {
			return out
		}
		infos, e := os.ReadDir(dir)
		if e != nil {
			fmt.Println("ReadDir error=>", e)
			return out
		}
		for _, info := range infos {
			if info.IsDir() {
				out = append(out, info.Name())
			}
		}
		return out
	})

	ipc.On("Unzip", func(sid, fpath string, distDir string) bool {
		go func() {
			eventName := sid
			err := Unzip(fpath, distDir, func(per1000 int) {
				socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "progress", Fva11: float32(per1000 / 100.0)})
			})
			if err != nil {
				socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "error", Error: err.Error()})
				return
			}
			socket.BroadMsgObj(&entity.NormMsg{Sub: eventName, Type: "succ"})
		}()
		return true
	})
	ipc.On("LocalAppProfile", func() map[string]string {
		config := conf.AppConfig
		exec, _ := os.Executable()
		_, exeName := filepath.Split(exec)

		return map[string]string{
			"wsPort":  fmt.Sprintf("%d", config.NatsWsPort),
			"version": Version,
			"dataDir": strings.ReplaceAll(config.DataDir, "\\", "/"),
			"exeDir":  config.ExeDir,
			"exeName": exeName,
			"exePath": exec,
		}
	})
}
