package api

import (
	"fmt"
	"launcher/bus"
	"time"

	"github.com/nats-io/nats.go"
	"infish.cn/comm"
)

type AssetListener struct {
	Id     string
	Type   string
	Action string
}

type AppListeners struct {
	Guid  string
	Title string
	List  []AssetListener
}

type ListenReq struct {
	Guid string
}
type AppInstReq struct {
	Type string
}
type AppOpenReq struct {
	Key string
}

var AppActionsMap = make(map[string]*AppListeners)

func StopApp(guid string) {
	fmt.Println("stoping", guid)
	if AppActionsMap[guid] != nil {
		delete(AppActionsMap, guid)
	}
	bus.NatsCenter.Publish("lancher.recv.change", []byte(guid))
}

func StartApp(guid string) {
	if AppActionsMap[guid] != nil {
		delete(AppActionsMap, guid)
	}
	bus.NatsCenter.Publish("lancher.recv.change", []byte(guid))
}

func RegAppMsgMgr() {
	changeApi := &comm.NatsMsgReplyer{
		Subject: "app.recv.change",
		Entity: func() interface{} {
			return &ListenReq{}
		},

		Cb: func(msg *nats.Msg, entity interface{}) interface{} {
			req, ok := entity.(*ListenReq)
			if !ok {
				return nil
			}

			if AppActionsMap[req.Guid] == nil {
				//获取对应的名称
				AppActionsMap[req.Guid] = &AppListeners{Guid: req.Guid, Title: GetAppName(req.Guid)}
			}

			//从应用身上重新拿取注册的所有行为
			err := bus.NatsCenter.Request("recv.actions."+req.Guid, nil, time.Second, &AppActionsMap[req.Guid].List)

			fmt.Println("recv.actions=>", err, AppActionsMap[req.Guid])

			//通知外面发送数据的应用窗口数据已经更新
			bus.NatsCenter.Publish("lancher.recv.change", []byte(req.Guid))

			return nil
		},
	}

	GetActionsApi := &comm.NatsMsgReplyer{
		Subject: "lancher.recv.actions",
		Entity: func() interface{} {
			return &ListenReq{}
		},
		Cb2: func(msg *nats.Msg, entity interface{}) (interface{}, error) {
			req, ok := entity.(*ListenReq)
			if !ok {
				return nil, nil
			}
			if AppActionsMap[req.Guid] == nil {
				AppActionsMap[req.Guid] = &AppListeners{}
			}
			return AppActionsMap[req.Guid], nil
		},
	}

	getAppInstApi := &comm.NatsMsgReplyer{
		Subject: "lancher.appInsts.get",
		Entity: func() interface{} {
			return &AppInstReq{}
		},
		Cb: func(msg *nats.Msg, entity interface{}) interface{} {
			req, ok := entity.(*AppInstReq)
			out := []*AppListeners{}
			if !ok {
				return out
			}
			for _, v := range AppActionsMap {

				app := &AppListeners{List: []AssetListener{}, Title: v.Title, Guid: v.Guid}

				for _, listen := range v.List {
					if listen.Type == req.Type {
						app.List = append(app.List, listen)
					}
				}
				if len(app.List) > 0 {
					out = append(out, app)
				}
			}
			return out
		},
	}

	bus.NatsCenter.AddReplyers(changeApi, GetActionsApi, getAppInstApi)
}
