package api

import (
	"fmt"
	"os"
	"os/user"
	"path/filepath"
	"strings"

	"image/png"
	_ "image/png"

	ico "github.com/Kodeworks/golang-image-ico"

	"github.com/energye/energy/v2/cef"
	"github.com/energye/energy/v2/cef/ipc"
	"github.com/go-ole/go-ole"
	"github.com/go-ole/go-ole/oleutil"
)

type ShortcutParams struct {
	AppName     string
	DesktopLink string
	LancherPath string
	IconPath    string
}

func RegShortcut(event *cef.BrowserEvent, window cef.IBrowserWindow, bw *cef.LCLBrowserWindow) {
	ipc.On("CreateShortcut", func(appName, lancherPath, icoPath string) bool {
		desktopLink, err := getDesktopLink(appName)
		if err != nil {
			fmt.Println(err)
			return false
		}
		err = createShortcut(&ShortcutParams{
			AppName:     appName,
			DesktopLink: desktopLink,
			LancherPath: lancherPath,
			IconPath:    icoPath,
		})
		if err != nil {
			fmt.Println(err)
			return false
		}
		return true
	})

	ipc.On("DeleteShortcut", func(appName string) bool {
		desktopLink, err := getDesktopLink(appName)
		if err != nil {
			fmt.Println(err)
			return false
		}
		err = os.Remove(desktopLink)
		if err != nil {
			fmt.Println(err)
			return false
		}
		return true
	})
}

func getDesktopLink(appName string) (string, error) {
	currentUser, err := user.Current()
	if err != nil {
		return "", err
	}
	_userDesktop := fmt.Sprintf("%s\\Desktop\\%s.lnk", currentUser.HomeDir, appName)
	userDesktop := strings.Replace(_userDesktop, "\\", "/", -1)
	return userDesktop, nil
}

// http://www.bathome.net/thread-33298-1-1.html
func createShortcut(para *ShortcutParams) error {
	ole.CoInitializeEx(0, ole.COINIT_APARTMENTTHREADED|ole.COINIT_SPEED_OVER_MEMORY)
	oleShellObject, err := oleutil.CreateObject("WScript.Shell")
	if err != nil {
		return err
	}
	defer oleShellObject.Release()
	wshell, err := oleShellObject.QueryInterface(ole.IID_IDispatch)
	if err != nil {
		return err
	}
	defer wshell.Release()
	cs, err := oleutil.CallMethod(wshell, "CreateShortcut", para.DesktopLink)
	if err != nil {
		return err
	}
	idispatch := cs.ToIDispatch()

	ext := filepath.Ext(para.IconPath)
	if ext == ".png" {
		iconPath, _ := strings.CutSuffix(para.IconPath, ext)
		icoFpath := fmt.Sprintf("%s.ico", iconPath)
		if IsExist(icoFpath) {
			para.IconPath = icoFpath
		} else { //根据当前ping创建一个
			pngFile, _ := os.Open(para.IconPath)
			if pngFile != nil {
				defer pngFile.Close()

				// 解码 PNG 图像
				pngImg, _ := png.Decode(pngFile)
				if pngImg != nil {

					// 创建 ICO 文件
					icoFile, _ := os.Create(icoFpath)
					if icoFile != nil {
						defer icoFile.Close()
						// 写入 ICO 文件
						err = ico.Encode(icoFile, pngImg)
						if err == nil {
							para.IconPath = icoFpath
						}
					}
				}
			}
		}
	}

	oleutil.PutProperty(idispatch, "TargetPath", para.LancherPath)
	oleutil.PutProperty(idispatch, "Arguments", fmt.Sprintf("-s %s", para.AppName))
	oleutil.PutProperty(idispatch, "IconLocation", para.IconPath)
	oleutil.CallMethod(idispatch, "Save")

	return nil
}
