package api

import (
	"fmt"
	"image"
	_ "image/jpeg"
	"path/filepath"

	"os"
	"strings"

	"github.com/disintegration/imaging"
	"github.com/energye/energy/v2/cef"
	"github.com/energye/energy/v2/cef/ipc"
	"github.com/energye/golcl/lcl"
	"github.com/energye/golcl/lcl/types"

	_ "image/png"
)

func RegSubWindows(event *cef.BrowserEvent, window cef.IBrowserWindow, bw *cef.LCLBrowserWindow) {

	var browserWindow *cef.LCLBrowserWindow
	ipc.On("openAssetsWindow", func(url, title string, width, height int32) bool {
		if browserWindow == nil || browserWindow.IsClosing() {
			wp := cef.NewWindowProperty()
			wp.Url = url
			wp.Title = title
			wp.EnableMinimize = true
			wp.EnableCenterWindow = true

			browserWindow = cef.NewLCLBrowserWindow(nil, wp)

			browserWindow.SetWidth(width)
			browserWindow.SetHeight(height)
			browserWindow.SetShowInTaskBar()
			browserWindow.DisableMaximize()
			// browserWindow.EnableDefaultCloseEvent()
			browserWindow.WorkAreaCenter()
			browserWindow.EnableAllDefaultEvent()

			browserWindow.Chromium().SetOnTitleChange(func(sender lcl.IObject, browser *cef.ICefBrowser, title string) {
				fmt.Println("SetOnTitleChange", wp.Title, title)
			})
		}
		cef.QueueAsyncCall(func(id int) {
			// browserWindow.SetFocus()
			browserWindow.Show()
			if browserWindow.WindowState() == types.WsMinimized {
				fmt.Println("WsMinimized windows")
				browserWindow.Restore()
			}
		})
		return true
	})

	ipc.On("OpenWeb", func(url string, title string, w float32, h float32) bool {

		wp := cef.NewWindowProperty()
		wp.Url = url
		wp.Title = title

		wp.EnableMinimize = true
		wp.EnableCenterWindow = true
		browser := cef.NewLCLBrowserWindow(nil, wp)
		width := int32(w)
		height := int32(h)
		if width == 0 {
			width = 1280
		}
		if height == 0 {
			height = 720
		}
		if w <= 1 {
			width = int32(float32(ScreenWidth) * w)
		}
		if h <= 1.0 {
			height = int32(float32(ScreenHeight) * h)
		}
		browser.SetWidth(width)
		browser.SetHeight(height)
		browser.SetShowInTaskBar()
		browser.WorkAreaCenter()
		browser.EnableAllDefaultEvent()

		fmt.Println("open web=>", url, title, width, height)

		cef.QueueAsyncCall(func(id int) {
			// browserWindow.SetFocus()
			browser.Show()
			if browser.WindowState() == types.WsMinimized {
				fmt.Println("WsMinimized windows")
				browser.Restore()
			}
		})

		return true
	})

	var browserQueen5 *cef.LCLBrowserWindow
	ipc.On("OpenQueen5", func(url, title string) bool {
		if browserQueen5 == nil || browserQueen5.IsClosing() {
			wp := cef.NewWindowProperty()
			wp.Url = url
			wp.Title = title

			wp.EnableMinimize = true
			wp.EnableCenterWindow = true

			browserQueen5 = cef.NewLCLBrowserWindow(nil, wp)

			browserQueen5.SetWidth(int32(float32(ScreenWidth) * 0.85))
			browserQueen5.SetHeight(int32(float32(ScreenHeight) * 0.85))

			browserQueen5.SetShowInTaskBar()
			// browserQueen5.DisableMaximize()
			// browserWindow.EnableDefaultCloseEvent()
			browserQueen5.WorkAreaCenter()
			browserQueen5.EnableAllDefaultEvent()

			browserQueen5.Chromium().SetOnTitleChange(func(sender lcl.IObject, browser *cef.ICefBrowser, title string) {
				fmt.Println("SetOnTitleChange", wp.Title, title)
			})
		}

		cef.QueueAsyncCall(func(id int) {
			// browserWindow.SetFocus()
			browserQueen5.Show()
			if browserQueen5.WindowState() == types.WsMinimized {
				fmt.Println("WsMinimized windows")
				browserQueen5.Restore()
			}
		})
		return true
	})

	var browserQueen5Play *cef.LCLBrowserWindow
	ipc.On("OpenQueen5Play", func(url, title string) bool {
		if browserQueen5Play == nil || browserQueen5Play.IsClosing() {
			wp := cef.NewWindowProperty()
			wp.Url = url
			wp.Title = title

			wp.EnableMinimize = true
			wp.EnableCenterWindow = true
			wp.EnableHideCaption = true

			browserQueen5Play = cef.NewLCLBrowserWindow(nil, wp)

			browserQueen5Play.SetWidth(int32(float32(ScreenWidth)))
			browserQueen5Play.SetHeight(int32(float32(ScreenHeight)))
			browserQueen5Play.SetShowInTaskBar()
			browserQueen5Play.EnableAllDefaultEvent()
			browserQueen5Play.SetOnKeyUp(func(sender lcl.IObject, key *types.Char, shift types.TShiftState) {
				fmt.Println("key=>", key, shift)
			})

			browserQueen5Play.WorkAreaCenter()
			browserQueen5Play.FullScreen()

			browserQueen5Play.Show()
		}

		cef.QueueAsyncCall(func(id int) {
			// browserWindow.SetFocus()
			browserQueen5Play.FullScreen()
			browserQueen5Play.Show()

			if browserQueen5Play.WindowState() == types.WsMinimized {
				fmt.Println("WsMinimized windows")
				browserQueen5Play.Restore()
			}
		})
		return true
	})

	ipc.On("SendWndMsg", func(msg string) bool {
		ipc.Emit("OnWndMsg", msg)
		return true
	})

	ipc.On("GetSaveFile", func(sid string, title string, filter string) bool {

		dlSave := lcl.NewSaveDialog(bw)
		dlSave.SetOptions(dlSave.Options().Include(types.OfShowHelp))

		dlSave.SetFilter(filter) //"文本文件(*.txt)|*.txt|所有文件(*.*)|*.*"
		dlSave.SetTitle(title)

		window.RunOnMainThread(func() {
			if dlSave.Execute() {
				fmt.Println("filename:=> ", dlSave.FileName())
				ipc.Emit(sid, dlSave.FileName())
				return
			}
			ipc.Emit(sid, "")
		})
		return true
	})

	ipc.On("OpenOneFile", func(sid string, title string, filter string) string {
		dlgOpen := lcl.NewOpenDialog(bw)
		dlgOpen.SetFilter(filter)
		dlgOpen.SetOptions(dlgOpen.Options().Include(types.OfShowHelp)) //rtl.Include(, types.OfAllowMultiSelect))
		dlgOpen.SetTitle(title)
		dlgOpen.Execute()
		name := dlgOpen.FileName()
		if len(name) > 0 {
			return strings.ReplaceAll(name, "\\", "/")
		}
		return name
	})

	ipc.On("GetImageMeta", func(fpath string) ImageMeta {
		out := ImageMeta{}
		inf, err := os.Stat(fpath)
		if err != nil {
			fmt.Println(err)
			return out
		}
		out.Size = inf.Size()

		if strings.ToLower(filepath.Ext(fpath)) == ".png" {
			// 打开图像文件
			img, err := imaging.Open(fpath)
			if err != nil {
				fmt.Println("无法打开图像文件:", err)
				return out
			}

			// 获取图像尺寸
			width := img.Bounds().Dx()
			height := img.Bounds().Dy()
			out.Width = width
			out.Height = height
			return out
		}

		file, err := os.Open(fpath)
		if err != nil {
			fmt.Println(err)
			return out
		}
		defer file.Close()

		fmt.Println("fpath")
		img, _, err := image.Decode(file)
		if err != nil {
			fmt.Println(err)
			return out
		}

		bounds := img.Bounds()
		out.Width = bounds.Dx()
		out.Height = bounds.Dy()

		return out
	})
}
