package api

import (
	"bytes"
	"fmt"
	"io"
	"launcher/conf"
	"net"
	"os"
	"path/filepath"

	"github.com/energye/energy/v2/pkgs/assetserve"
)

var ScreenWidth = 0
var ScreenHeight = 0

func CopyFile(src string, target string) bool {
	dir := filepath.Dir(target)
	os.MkdirAll(dir, os.ModePerm)

	// 打开源文件
	file1, err1 := os.Open(src)
	if err1 != nil {
		fmt.Println("[CopyFile]", "打开源文件", src, "失败", err1)
		return false
	}
	// 创建目标文件
	file2, err2 := os.OpenFile(target, os.O_RDWR|os.O_CREATE, os.ModePerm)
	if err2 != nil {
		fmt.Println("[CopyFile]", "打开目标文件", src, "失败", err2)
		return false
	}
	//使用结束关闭文件
	defer file1.Close()
	defer file2.Close()
	n, e := io.Copy(file2, file1)
	if e != nil {
		fmt.Println(e)
		return false
	}
	fmt.Println("[CopyFile] succ", n)
	return true
}

func IsDir(path string) bool {
	s, err := os.Stat(path)
	if err != nil {

		return false
	}
	return s.IsDir()

}

func IsZipFile(data []byte) bool {
	if len(data) >= 4 {
		return bytes.Equal(data[:4], []byte("PK\x03\x04"))
	}
	return false
}

var prjHttpServers map[string]string = make(map[string]string)

func GetFreePort() (int, error) {
	addr, err := net.ResolveTCPAddr("tcp", "localhost:0")
	if err != nil {
		return 0, err
	}

	l, err := net.ListenTCP("tcp", addr)
	if err != nil {
		return 0, err
	}
	defer l.Close()
	return l.Addr().(*net.TCPAddr).Port, nil
}

func startProjectHttp(dir string) string {
	if len(prjHttpServers[dir]) > 0 {
		return prjHttpServers[dir]
	}

	p, e := GetFreePort()
	if e != nil {
		fmt.Println("获取端口失败", e.Error())
		return ""
	}

	serverAppData := assetserve.NewAssetsHttpServer()
	serverAppData.PORT = p
	serverAppData.LocalAssets = dir + "/" //必须设置目录名

	ip := conf.AppConfig.Ip
	if len(ip) < 1 {
		ip = "localhost"
	}
	prjHttpServers[dir] = fmt.Sprintf("http://%s:%d/", ip, p)

	go serverAppData.StartHttpServer()

	return prjHttpServers[dir]
}

func GetDirSize(dir string) int64 {
	totalSize := int64(0)
	// 遍历目录及其子目录下的所有文件
	err := filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
		if err != nil {
			fmt.Printf("访问文件或目录时出错：%v\n", err)
			return nil
		}
		// 只统计文件大小，忽略目录
		if !info.IsDir() {
			totalSize += info.Size()
		}
		fmt.Println(totalSize)
		return nil
	})
	if err != nil {
		fmt.Printf("遍历目录时出错：%v\n", err)
	}
	return totalSize
}
