package api

import (
	"archive/zip"
	"errors"
	"fmt"
	"io"
	"launcher/huawei"
	"net/http"
	"os"
	"os/exec"
	"path"
	"path/filepath"
	"runtime"
	"strings"
	"time"

	"github.com/energye/energy/v2/cef"
	"github.com/energye/energy/v2/cef/ipc"
)

type QuitAppFunc = func(flag int)

var QuitApp QuitAppFunc

func RegUpgrade(event *cef.BrowserEvent, window cef.IBrowserWindow, bw *cef.LCLBrowserWindow) {
	ipc.On("Upgrade", func(updaterUrl string, lancherUrl string) bool {
		go func() {
			// 下载文件
			appExePath, _ := os.Executable()
			appExePath = strings.Replace(appExePath, "\\", "/", -1)
			appExeDir, _ := filepath.Split(appExePath)
			// 删除已有的updater.exe
			updaterPath := fmt.Sprintf("%supdater.exe", appExeDir)
			os.Remove(updaterPath)
			// 下载updater.exe到当前目录
			// updaterUrl = "http://spu3dv1.obs.cn-east-3.myhuaweicloud.com/pkg/updater1.0.2.windows-amd64.zip"
			// lancherUrl = "http://spu3dv1.obs.cn-east-3.myhuaweicloud.com/pkg/lancher1.0.2.windows-amd64.zip"
			err := DownloadExe(updaterPath, updaterUrl)
			if err != nil {
				fmt.Println(err)
				return
			}
			fmt.Println("---natsport", NatsPort)
			time.Sleep(2 * time.Second)
			// 带参数执行updater
			cmd := exec.Command(updaterPath, fmt.Sprintf("--np=%d", NatsPort), fmt.Sprintf("--url=%s", lancherUrl))
			cmd.Stdout = os.Stdout
			cmd.Stderr = os.Stderr
			err = cmd.Start()
			if err != nil {
				fmt.Printf("failed to call cmd.Start(): %v", err)
				return
			}

			// // 关闭窗口
			// StopRunningApp()

			// if QuitApp != nil {
			// 	QuitApp(2)
			// }
		}()

		return true
	})
}

func DownloadExe(appExe, url string) error {
	fmt.Println("download url => ", url)
	resp, err := http.Get(url)
	if err != nil {
		return err
	}
	defer resp.Body.Close()
	if path.Ext(url) == ".zip" {
		fmt.Println("not zip")
		bytes, err := io.ReadAll(resp.Body)
		if err != nil {
			return err
		}
		appExeDir, _ := filepath.Split(appExe)
		err = UnzipByte(bytes, appExeDir, appExe)
		if err != nil {
			return err
		}
		return nil
	}

	fmt.Println("not zip")
	// 创建一个文件用于保存
	out, err := os.Create(appExe)
	if err != nil {
		return err
	}
	defer out.Close()

	// 然后将响应流和文件流对接起来
	_, err = io.Copy(out, resp.Body)
	if err != nil {
		return err
	}
	return nil
}

func UnzipByte(zipData []byte, dstDir string, appexe string) error {
	total := len(zipData)
	zipReader, err := zip.NewReader(strings.NewReader(string(zipData)), int64(total))
	if err != nil {
		return err
	}
	for _, file := range zipReader.File {

		if file.FileInfo().IsDir() {
			err := os.MkdirAll(file.Name, 0644)
			if err != nil {
				fmt.Println(err)
			}
			continue
		}
		//为文件时，打开文件
		zipFile, err := file.Open()

		//文件为空的时候，打印错误
		if err != nil {
			fmt.Println(err)
			continue
		}
		//这里在控制台输出文件的文件名及路径
		fmt.Println("unzip: ", file.Name)

		//在对应的目录中创建相同的文件
		NewFile, err := os.Create(appexe)
		if err != nil {
			fmt.Println(err)
			continue
		}
		//将内容复制
		io.Copy(NewFile, zipFile)
		//关闭文件
		NewFile.Close()
		zipFile.Close()

	}
	return nil
}

// 上传当前exe文件
func UploadExe() (string, error) {
	// 获取当前执行程序
	// G:\wk\lancher-svc\src\lancher.exe
	appExePath, _ := os.Executable()
	// G:/wk/lancher-svc/src/lancher.exe
	appExePath = strings.Replace(appExePath, "\\", "/", -1)
	// G:/wk/lancher-svc/src/
	appExeDir, _ := filepath.Split(appExePath)
	fmt.Println(appExeDir)

	// 压缩当前执行程序
	// 压缩单个文件
	// 压缩文件名 lancher1.0.2.windows-amd64.zip
	zipFile := fmt.Sprintf("%s%s.%s-%s.zip", "lancher", Version, runtime.GOOS, runtime.GOARCH)
	zipPath := fmt.Sprintf("%s%s", appExeDir, zipFile)
	err := ZipFile(zipPath, appExePath)
	if err != nil {
		return "", err
	}
	// 获取操作系统和版本号确定上传文件名
	obsDir := "pkg"
	//上传压缩后的文件到obs
	// http://spu3dv1.obs.cn-east-3.myhuaweicloud.com/pkg/lancher1.0.2.windows-amd64.zip
	huawei.InitConfig()
	obs, err := huawei.UploadFile(zipPath, obsDir, zipFile)
	if err != nil {
		return "", err
	}
	if len(obs.Url) < 1 {
		fmt.Println("上传zipfile错误")
		return "", errors.New("上传zipfile错误")
	}
	fmt.Println("succ uploaded=>", obs.Url)

	return zipPath, nil

}

func ZipFile(zipPath string, exePath string) error {
	os.RemoveAll(zipPath)
	archive, err := os.Create(zipPath)
	if err != nil {
		return err
	}
	defer archive.Close()
	zipWriter := zip.NewWriter(archive)

	f, err := os.Open(exePath)
	if err != nil {
		return err
	}
	defer f.Close()
	_, exeFile := filepath.Split(exePath)

	w, err := zipWriter.Create(exeFile)
	if err != nil {
		return err
	}
	if _, err := io.Copy(w, f); err != nil {
		return err
	}

	zipWriter.Close()
	return nil
}
