package api

import (
	"fmt"
	"launcher/huawei"
	"os"
	"path/filepath"
	"strings"
	"sync"

	"github.com/energye/energy/v2/cef"
	"github.com/energye/energy/v2/cef/ipc"
)

type UploadItem struct {
	Id    string `json:"id"`
	Fpath string `json:"fpath"`
	Dir   string `json:"dir"`
	Name  string `json:"name"`
	Url   string `json:"url"`
	Size  int64  `json:"size"`
}

func RegUpload(event *cef.BrowserEvent, window cef.IBrowserWindow, bw *cef.LCLBrowserWindow) {

	//同步上传
	ipc.On("UploadSync", func(items []*UploadItem) []*UploadItem {
		count := len(items)
		if count < 1 {
			return items
		}

		var wg sync.WaitGroup
		wg.Add(count)

		// 启动并发的任务
		for i := 0; i < count; i++ {
			go func(taskNum int) {
				item := items[taskNum]
				fmt.Println("uplading ", item.Fpath)
				ret, err := huawei.UploadFile(item.Fpath, item.Dir, item.Name)
				if err == nil {
					item.Url = ret.Url
					item.Size = ret.Size
				}
				fmt.Println("uplading over=>", item.Fpath)
				wg.Done()
			}(i)
		}
		// 等待所有任务执行完成
		wg.Wait()

		return items
	})

	//异步上传
	ipc.On("UploadASync", func(sid string, items []*UploadItem) bool {
		go func() {
			count := len(items)
			if count < 1 {
				ipc.Emit(sid, items)
				return
			}

			var wg sync.WaitGroup
			wg.Add(count)

			// 启动并发的任务
			for i := 0; i < count; i++ {
				go func(taskNum int) {
					item := items[taskNum]
					fmt.Println("uplading ", item.Fpath)
					ret, err := huawei.UploadFile(item.Fpath, item.Dir, item.Name)
					if err == nil {
						item.Url = ret.Url
						item.Size = ret.Size
					}
					fmt.Println("uplading over=>", item.Fpath)
					wg.Done()
				}(i)
			}
			// 等待所有任务执行完成
			wg.Wait()
			ipc.Emit(sid, items)
		}()
		return true
	})

	//异步上传
	ipc.On("UploadDir", func(sid string, dir string, targetDir string) bool {
		items := []*UploadItem{}
		hasErr := false

		dir = strings.ReplaceAll(dir, "\\", "/")

		// 遍历目录及其子目录下的所有文件
		err := filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
			if err != nil {
				fmt.Printf("访问文件或目录时出错：%v\n", err, path)
				hasErr = true
				return nil
			}
			// 只统计文件大小，忽略目录
			if !info.IsDir() {
				filePath := strings.ReplaceAll(path, "\\", "/")
				fileDir := filepath.Dir(path)
				fileDir = strings.ReplaceAll(fileDir, "\\", "/")
				subdir, _ := strings.CutPrefix(fileDir, dir)
				items = append(items, &UploadItem{Id: fmt.Sprintf("%d", len(items)), Fpath: filePath, Dir: targetDir + subdir})
			}
			return nil
		})
		if err != nil || hasErr {
			fmt.Printf("遍历目录时出错：%v\n", err)
			return false
		}

		count := len(items)
		if count < 1 {
			return false
		}

		go func() {

			var wg sync.WaitGroup
			wg.Add(count)

			// 启动并发的任务
			for i := 0; i < count; i++ {
				go func(taskNum int) {
					item := items[taskNum]
					ret, err := huawei.UploadFile(item.Fpath, item.Dir, item.Name)
					if err == nil {
						item.Url = ret.Url
						item.Size = ret.Size
					}
					wg.Done()
				}(i)
			}
			// 等待所有任务执行完成
			wg.Wait()
			ipc.Emit(sid, items)
		}()

		return true
	})
}
