package main

import (
	"embed"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"launcher/api"
	"launcher/bus"
	"launcher/conf"
	"os"
	"path"
	"path/filepath"
	"runtime"
	"strings"
	"time"

	"github.com/energye/energy/v2/cef"
	"github.com/energye/energy/v2/cef/ipc"
	"github.com/energye/energy/v2/common"
	"github.com/energye/energy/v2/consts"
	"github.com/energye/golcl/lcl"
	"github.com/energye/golcl/lcl/types"
	"github.com/jessevdk/go-flags"
	"infish.cn/comm"

	"github.com/kardianos/osext"
)

//go:embed assets
var resources embed.FS

type SysBusProfile struct {
	NatsWsPort int
	NatsPort   int
}

var AppDataDir = "./data"
var GAppOption = &AppOption{}

// short只能是一个字符
type AppOption struct {
	Shortcut string `short:"s" long:"shortcut" description:"conf shortcut"`
	Upload   string `short:"u" long:"upload" description:"upload exe zip file"`
}

func (o *AppOption) Parse() {
	_, err := flags.NewParser(o, flags.Default|flags.IgnoreUnknown).Parse()
	fmt.Println(err)
}

// Will return true if the process with PID exists.
// func checkPid(pid int) bool {
// 	p, err := os.FindProcess(pid)
// 	if err != nil {
// 		return false
// 	}
// 	return p != nil
// }

type ResumFuc = func()

var ResumMainWnd ResumFuc
var MainUrl = fmt.Sprintf("http://spu3d.com/lancher/v%s/lancher.html", api.Version)

//go:generate goversioninfo
func main() {
	GAppOption.Parse()

	isCefSubPross := false
	for _, para := range os.Args {
		if strings.Contains(para, "--type=") {
			isCefSubPross = true
			fmt.Println(para)
			break
		}
	}
	fmt.Println("isCefSubPross=>", isCefSubPross, GAppOption)

	// 上传编译好的执行文件
	if len(GAppOption.Upload) > 0 {
		zipPath, err := api.UploadExe()
		if err != nil {
			fmt.Println(err)
		}
		// 成功后删除压缩文件
		os.Remove(zipPath)
		return
	}
	f, _ := osext.ExecutableFolder()
	ExeDir, _ := os.Getwd()

	if strings.LastIndex(f, "\\go-build") == -1 {
		ExeDir = f
	}

	fmt.Println("exeDir=>", ExeDir, runtime.GOOS, runtime.GOARCH)
	AppDataDir = path.Join(ExeDir, "data")
	AppDataDir = strings.ReplaceAll(AppDataDir, "\\", "/")

	fmt.Println("AppDataDir=>", AppDataDir)

	p1, p2, e := GenHttpPort()
	if e != nil {
		fmt.Println("获取端口出错")
		return
	}
	api.NatsPort = p2
	conf.AppConfig = &conf.AppConf{
		Configer:    []*conf.ConfigerItem{},
		Adapter:     []*conf.ProxyItem{},
		DataDir:     AppDataDir,
		ExeDir:      strings.ReplaceAll(ExeDir, "\\", "/"),
		NatsApiPort: p2,
		NatsWsPort:  p1,
		Ip:          "localhost",
		Nats:        conf.Nats{Url: fmt.Sprintf("nats://localhost:%d", p2)},
	}

	//判断主线程是否已经启动过
	//判断是否是cef子进程
	//启动ui管理窗口
	cefapp := createRunUi(MainUrl)
	if isCefSubPross {
		//判断主线程是否已经启动过
		//判断是否是cef子进程
		//启动ui管理窗口
		cef.Run(cefapp)
		fmt.Println("CefSubPross quited")
		return
	}

	pid, _ := os.ReadFile(path.Join(ExeDir, "pid"))
	if len(pid) > 0 {
		remoteCnn, _ := comm.NewNatsBus(fmt.Sprintf("nats://localhost:%d", p2), 1, 1, []*comm.NatsStreamWather{})
		if remoteCnn != nil {
			//已经启动过了，发送页面显示消息
			d, _ := json.Marshal(map[string]string{"shortcut": GAppOption.Shortcut})
			remoteCnn.Publish("lancher.resume", d)

			remoteCnn.GetNatsConn().Flush()
			return
		}
	}

	os.WriteFile(path.Join(ExeDir, "pid"), []byte(fmt.Sprintf("%d", os.Getpid())), os.ModePerm)

	//先启动nats
	e = gNatsMrg.InitRun(p1, p2)
	if e != nil {
		fmt.Println(e)
		return
	}

	b, adapters := bus.NewNatsBus(conf.AppConfig)
	for _, remote := range adapters {
		if remote != nil {
			go remote.Run(nil)
		}
	}
	if e != nil {
		fmt.Println("add db error", e)
		return
	}
	api.RegAppMsgMgr()

	go b.Run(func() {
		// wg.Wait()
		p := &SysBusProfile{NatsWsPort: p1, NatsPort: p2}
		d, _ := json.Marshal(p)
		bus.NatsCenter.SetConfig("nats.profile", string(d))

		fmt.Printf("natsport=>%d wsport=>%d \n", p2, p1)

		fmt.Println("lancher started")
		fmt.Println("lancher is running")
		// go tray.StartSysTray()
	})
	api.StartWebSock()

	cef.Run(cefapp)

	api.StopRunningApp()

	quitRun()

	os.Remove(path.Join(ExeDir, "pid"))

	// if QuitingFlag == 2 {
	// 	isQuiting = false
	// 	exePath, _ := os.Executable()
	// 	fmt.Println("need restart ", exePath)
	// 	// 重启
	// 	cmd := exec.Command(exePath)
	// 	cmd.Stdout = os.Stdout
	// 	cmd.Stderr = os.Stderr
	// 	err := cmd.Start()
	// 	if err != nil {
	// 		fmt.Printf("failed to call cmd.Start(): %v", err)
	// 	}
	// 	os.WriteFile(path.Join(ExeDir, "pid"), []byte(fmt.Sprintf("%d", cmd.Process.Pid)), fs.FileMode(os.O_TRUNC))
	// 	log.Printf("pid: %d", cmd.Process.Pid)
	// 	lancherbak := fmt.Sprintf("%s/%s", ExeDir, "lancherlast")
	// 	if api.IsExist(lancherbak) {
	// 		err := os.Remove(lancherbak)
	// 		if err != nil {
	// 			fmt.Println(err)
	// 		}
	// 	}
	// 	cmd.Process.Wait()
	// }
}

// type AppJson struct {
// 	Version        string
// 	LastUpdateDate string
// 	Icon           string
// 	// RevAssets      string
// 	Size      string
// 	Tags      string
// 	Desc      string
// 	Entry     string
// 	Type      string
// 	Key       string
// 	InstCount int
// }

func quitRun() {
	if bus.NatsCenter != nil {
		bus.NatsCenter.Quit()
	}
	gNatsMrg.Quit()
}

func createRunUi(url string) *cef.TCEFApplication {
	//全局初始化 每个应用都必须调用的
	cef.GlobalInit(nil, &resources)
	//创建应用
	var app = cef.NewApplication()
	fmt.Println("cef-------------")
	// app.SetSingleProcess(true)

	app.SetAllowFileAccessFromFiles(true)
	app.SetDisableWebSecurity(true)
	app.SetAllowInsecureLocalhost(true)
	app.SetEnableGPU(true)
	app.SetAllowUniversalAccessFromFileUrls(true)
	app.SetAllowRunningInsecureContent(true)
	app.SetAutoplayPolicy(consts.AppNoUserGestureRequired)
	app.SetMuteAudio(false)
	app.SetCommandLineArgsDisabled(false)
	app.SetDisableJavascriptAccessClipboard(false)

	shortcut := "self"
	if len(GAppOption.Shortcut) > 0 {
		shortcut = GAppOption.Shortcut
	}
	fmt.Println("short cut=>", shortcut)

	urlpath := filepath.Join(conf.AppConfig.ExeDir, "url")
	isUrl := api.IsExist(urlpath)
	if isUrl {
		uri, e := os.ReadFile(urlpath)
		if e == nil && len(uri) > 1 {
			url = string(uri)
		}
	}
	cef.BrowserWindow.Config.Url = fmt.Sprintf("%s?t=%d&shortcut=%s", url, time.Now().UnixMicro(), shortcut)
	cef.BrowserWindow.Config.Title = "lancher"
	cef.BrowserWindow.Config.IconFS = "assets/3dqueen.ico"
	// !这里有问题，minHeight 不正确,对应宽度且高度设置不生效/minWidth
	cef.BrowserWindow.Config.MinHeight = 380
	cef.BrowserWindow.Config.MinWidth = 380
	cef.BrowserWindow.Config.Width = 1280
	cef.BrowserWindow.Config.Height = 720
	// 窗口固定不可调整
	// cef.BrowserWindow.Config.EnableResize = false
	// 取消windows菜单栏
	cef.BrowserWindow.Config.EnableHideCaption = true

	app.SetCache(filepath.Join(AppDataDir, "cache"))

	// 注册自定义scheme
	app.SetOnRegCustomSchemes(func(registrar *cef.TCefSchemeRegistrarRef) {
		registrar.AddCustomScheme("file", consts.CEF_SCHEME_OPTION_STANDARD|consts.CEF_SCHEME_OPTION_LOCAL)
	})

	// 在浏览器窗口初始化回调中注册IPC事件

	cef.BrowserWindow.SetBrowserInit(func(event *cef.BrowserEvent, window cef.IBrowserWindow) {

		screen := window.Screen()
		pr := screen.Primary()
		api.ScreenHeight = int(pr.WorkArea.Height)
		api.ScreenWidth = int(pr.WorkArea.Width)

		window.Chromium().SetWebRTCIPHandlingPolicy(consts.HpDefaultPublicAndPrivateInterfaces)
		window.Chromium().SetWebRTCMultipleRoutes(consts.STATE_ENABLED)
		window.Chromium().SetWebRTCNonproxiedUDP(consts.STATE_ENABLED)

		fmt.Println("WebRTCIPHandlingPolicy=>", window.Chromium().WebRTCIPHandlingPolicy())

		fmt.Println("WebRTCMultipleRoutes=>", window.Chromium().WebRTCMultipleRoutes())
		fmt.Println("WebRTCNonproxiedUDP=>", window.Chromium().WebRTCNonproxiedUDP())

		api.RegDrag(event)

		event.SetOnBeforeDownload(func(sender lcl.IObject, browser *cef.ICefBrowser, downloadItem *cef.ICefDownloadItem, suggestedName string, callback *cef.ICefBeforeDownloadCallback, window cef.IBrowserWindow) {
			// 对字符串进行Base64解码
			total := strings.LastIndex(suggestedName, ".")
			decodedBytes, err := base64.StdEncoding.DecodeString(suggestedName[:total])
			if err != nil {
				fmt.Println("解码错误:", err)
				return
			}
			decodedStr := string(decodedBytes)
			decodedStr = strings.ReplaceAll(decodedStr, "/", consts.Separator)
			fmt.Println("xx=>", decodedStr)
			callback.Cont(decodedStr, false)
		})

		//下载更新事件
		//1. 返回下载进度
		//2. downloadItem 下载项
		//3. callback 下载状态的控制, 下载暂停，开始、取消
		//4. 将下载进度通过事件机制发送到html中展示
		event.SetOnDownloadUpdated(func(sender lcl.IObject, browser *cef.ICefBrowser, downloadItem *cef.ICefDownloadItem, callback *cef.ICefDownloadItemCallback) {
			//传递数据参数到html中
			//这些参数按下标顺序对应到js函数参数位置
			//演示只传递了几个参数
			fmt.Println(downloadItem.State(), downloadItem.TotalBytes(), "/", downloadItem.ReceivedBytes(), "fullPath", downloadItem.FullPath())
			if downloadItem.State() == 2 {
				ipc.Emit("downloadsucc", strings.ReplaceAll(downloadItem.FullPath(), "\\", "/"), downloadItem.ReceivedBytes())
			}
		})

		if window.IsLCL() {
			//LCL 窗口是我们创建的，需要我们自己管理窗口
			bw := window.AsLCLBrowserWindow().BrowserWindow()
			bw.SetOnClose(func(sender lcl.IObject, action *types.TCloseAction) bool {

				if isQuiting {
					// *action = types.CaFree
					return false
				}

				*action = types.CaHide //隐藏窗口
				return true            //跳过默认事件, 如果想关闭窗口，这里返回false
			})
			bw.SetOnCloseQuery(func(sender lcl.IObject, canClose *bool) bool {
				return true //跳过默认事件, 如果想关闭窗口，这里返回false
			})

			trayDemo(window)

			ipc.On("ResumeMainWnd", func() bool {
				bw.SetVisible(true)
				bw.RunOnMainThread(func() {
					bw.SetFocus()
				})
				return true
			})

			ipc.On("HideMainWnd", func() bool {
				bw.SetVisible(false)
				return true
			})
			ipc.On("QuitLancher", func() bool {
				isQuiting = true
				window.CloseBrowserWindow()
				return true
			})
			var setMinWindow = func() {
				window.SetWidth(380)
				window.SetHeight(720)
			}
			var setDefaultWindow = func() {
				window.SetWidth(1280)
				window.SetHeight(720)
			}

			// 最大化窗口 参数bool
			ipc.On("MaxWindow", func(isMax bool) bool {
				if isMax {
					window.Maximize()
				} else {
					window.Restore()
				}
				return true
			})

			// 调整窗口 参数bool
			ipc.On("FoldWindow", func(isfold bool) bool {
				window.Restore()
				time.Sleep(80 * time.Millisecond)
				if isfold {
					// window.EnableResize()
					setMinWindow()
				} else {
					setDefaultWindow()
				}
				return true
			})

		} else if window.IsViewsFramework() {
			//VF 窗口是CEF自己创建的，这里我们只管Chromium的Close事件即可
			bw := window.AsViewsFrameworkBrowserWindow().BrowserWindow()
			bw.SetOnCloseQuery(func(sender lcl.IObject, cefWindow *cef.ICefWindow, window cef.IBrowserWindow, canClose *bool) bool {
				if isQuiting {
					*canClose = true
					return true
				}
				*canClose = false //取消关闭 , 如果想关闭窗口 true
				window.Hide()     //隐藏窗口
				return true       //跳过默认事件, 如果想关闭窗口，这里返回false
			})
		}

		// 页面加载完成后 发送消息, 页面 radio 选中
		window.Chromium().SetOnLoadStart(func(sender lcl.IObject, browser *cef.ICefBrowser, frame *cef.ICefFrame, transitionType consts.TCefTransitionType) {
			fmt.Println("OnLoadStart ===============")
			RegSchema(browser)
		})

		// 系统消息提示框目前仅能在LCL窗口组件下使用
		// LCL 各种系统组件需要在UI线程中执行, 但ipc.on非UI线程
		// 所以需要使用 QueueAsyncCall 包裹在UI线程中执行
		if window.IsLCL() { // LCL window from
			bw := window.AsLCLBrowserWindow().BrowserWindow()

			dlPageSetupDialog := lcl.NewPageSetupDialog(bw)
			dlPrinterSetupDialog := lcl.NewPrinterSetupDialog(bw)
			dlSelDirdlg := lcl.NewSelectDirectoryDialog(bw)

			var (
				dlPicSave *lcl.TSavePictureDialog
				dlPicOpen *lcl.TOpenPictureDialog
			)
			// VF 窗口，在linux不能使用它，还不知道是什么原因...
			if !common.IsLinux() && !app.IsMessageLoop() {
				dlPicSave = lcl.NewSavePictureDialog(bw)
				dlPicOpen = lcl.NewOpenPictureDialog(bw)
			}

			dlColor := lcl.NewColorDialog(bw)

			dlFont := lcl.NewFontDialog(bw)

			dlgOpen := lcl.NewOpenDialog(bw)
			dlgOpen.SetFilter("文本文件(*.txt)|*.txt|所有文件(*.*)|*.*")
			//    dlgOpen.SetInitialDir()
			//	dlgOpen.SetFilterIndex()
			dlgOpen.SetOptions(dlgOpen.Options().Include(types.OfShowHelp, types.OfAllowMultiSelect)) //rtl.Include(, types.OfShowHelp))
			dlgOpen.SetTitle("打开")

			api.RegApi(event, window, bw)

			ipc.On("showDialog", func(t int) {
				window.RunOnMainThread(func() {
					// fmt.Println("call-1-DMainThreadId:", api.DMainThreadId(), api.DCurrentThreadId())
					//cef.QueueAsyncCall(func(id int) {
					switch t {
					case 1: // InputComboEx
						fmt.Println(lcl.InputComboEx("选择", "请选择一项："+strings.Repeat(" ", 50), []string{"第一项", "第二项", "第三项", "第四项"}, false))
					case 2: // InputCombo
						fmt.Println(lcl.InputCombo("选择", "请选择一项："+strings.Repeat(" ", 50), []string{"第一项", "第二项", "第三项", "第四项"}))
					case 3: // PasswordBox
						fmt.Println(lcl.PasswordBox("输入", "请输入密码："))
					case 4: // PageSetup Dialog
						dlPageSetupDialog.Execute()
					case 5: // PrinterSetup Dialog
						dlPrinterSetupDialog.Execute()
					case 6: // InputQuery
						s := "default"
						if lcl.InputQuery("标题", "提示", &s) {
							fmt.Println("结果：", s)
						}
					case 7: // InputBox
						s := lcl.InputBox("标题", "提示", "默认值")
						if s != "" {
							fmt.Println("结果：", s)
						}
					case 10: // SelectDirectory1
						if ok, dir := lcl.SelectDirectory1(0); ok {
							fmt.Println("选择的目录为：", dir)
						}
					case 11: // SelectDirectory2
						if ok, dir := lcl.SelectDirectory2("标题了", "C:/", true); ok {
							fmt.Println("选择的目录为：", dir)
						}
					case 12: // Select Directory Dialog
						if dlSelDirdlg.Execute() {
							fmt.Println("Name: ", dlSelDirdlg.FileName())
						}
					case 13: // SavePic Dialog
						if dlPicSave != nil && dlPicSave.Execute() {
							fmt.Println("Name: ", dlPicSave.FileName())
						}
					case 14: // OpenPic Dialog
						if dlPicOpen != nil && dlPicOpen.Execute() {
							fmt.Println("Name: ", dlPicOpen.FileName())
						}
					case 15: // Color Dialog
						if dlColor.Execute() {
							fmt.Println("Color: ", dlColor.Color())
						}
					case 16: // Font Dialog
						if dlFont.Execute() {
							fmt.Println("Name: ", dlFont.Font().Name())
						}

					case 18: // Open Dialog
						if dlgOpen.Execute() {
							fmt.Println("filename: ", dlgOpen.FileName())
						}
					}
				})
			})

		} else { // VF window
			// 可配合 Chromium().SetOnFileDialog 回调函数使用
			//window.Chromium().SetOnFileDialog(func(sender lcl.IObject, browser *cef.ICefBrowser, mode consts.FileDialogMode, title, defaultFilePath string, acceptFilters *lcl.TStrings, callback *cef.ICefFileDialogCallback) bool {
			//	fmt.Println("Chromium SetOnFileDialog", mode, title, defaultFilePath, "acceptFilters:", acceptFilters.Count())
			//	acceptFilters.Add(".png")
			//	callback.Cont([]string{"/file/to/path/file.xx"}) // 设置选择的文件
			//	return true
			//})
			// 定义 dialog 回调函数
			callback := cef.RunFileDialogCallbackRef.New()
			callback.SetOnFileDialogDismissed(func(filePaths *lcl.TStrings) {
				for i := 0; i < int(filePaths.Count()); i++ {
					path := filePaths.Strings(int32(i))
					fmt.Println(path)
				}
			})
			ipc.On("showDialog", func(t int) {
				switch t {
				case 1:
					window.Chromium().Browser().RunFileDialog(consts.FILE_DIALOG_OPEN_FOLDER, "打开文件夹", "", nil, callback)
				case 2:
					acceptFilters := lcl.NewStringList()
					acceptFilters.Add(".png")
					window.Chromium().Browser().RunFileDialog(consts.FILE_DIALOG_SAVE, "保存图片", "", acceptFilters, callback)
				case 3:
					acceptFilters := lcl.NewStringList()
					acceptFilters.Add(".png")
					window.Chromium().Browser().RunFileDialog(consts.FILE_DIALOG_OPEN, "打开图片", "", acceptFilters, callback)
				case 4:
					window.Chromium().Browser().RunFileDialog(consts.FILE_DIALOG_SAVE, "打开", "", nil, callback)
				case 5:
					window.Chromium().Browser().RunFileDialog(consts.FILE_DIALOG_OPEN, "保存", "", nil, callback)
				}
			})
		}

	})

	return app
}

var isQuiting = false
var QuitingFlag = 0

// LCL组件托盘, 适用windows和macosx
// 目前 不支持linux因GTK2和GTK3共存问题, 以后解决GTK3问题将支持
// 或linux cef 106以前的版本使用GTK2的的也可使用该托盘
func trayDemo(browserWindow cef.IBrowserWindow) {
	window := browserWindow.AsLCLBrowserWindow().BrowserWindow()
	//托盘 windows linux macos 系统托盘
	newTray := window.NewTray()
	newTray.SetTitle("3dqueen")
	newTray.SetHint("3dqueen")
	// if common.IsLinux() {
	// 	newTray.SetIconFS("assets/icon.png")
	// } else {
	// 	newTray.SetIconFS("assets/icon.ico")
	// }
	tray := newTray.AsLCLTray()
	// menu1 := tray.AddMenuItem("父菜单", nil)
	// //带图标的菜单
	// iconItem := tray.NewMenuItem("带个图标", nil)
	// iconItem.Bitmap().SetSize(32, 32)      //图标情况调整大小
	// iconItem.Bitmap().SetTransparent(true) //透明
	// icon := lcl.NewIcon()
	// icon.LoadFromFSFile("assets/icon_1.ico")
	// iconItem.Bitmap().Canvas().Draw(0, 0, icon) //画上去
	// tray.TrayMenu().Items().Add(iconItem)

	tray.SetOnClick(func() {
		fmt.Println("click")
		bus.NatsCenter.Publish("lancher.trayclicked", []byte{})
	})

	// tray.SetOnMouseDown(func(sender lcl.IObject, button lcl., shift lcl.TShiftState, x, y int32) {
	// 	fmt.Println("SetOnMouseDown")
	// })

	ipc.On("TrayNotice", func(title, context string, timeout int32) bool {
		tray.Notice(title, context, timeout)
		return true
	})

	// ipc.On("UpdateTrayMenu", func(title, context string, timeout int32) bool {
	// 	tray.Notice(title, context, timeout)
	// 	return true
	// })

	ResumMainWnd = func() {
		window.SetVisible(true)
		window.RunOnMainThread(func() {
			window.SetFocus()
		})
	}

	tray.AddMenuItem("显示/隐藏", func() {
		window.SetVisible(!window.Visible())
		if window.Visible() { //之后的显示状态
			window.RunOnMainThread(func() {
				window.SetFocus()
			})
		}
	})
	tray.AddMenuItem("退出", func() {
		err := bus.NatsCenter.Publish("lancher.quitclicked", []byte{})
		if err != nil {
			isQuiting = true
			browserWindow.CloseBrowserWindow()
		}
	})

	api.QuitApp = func(flag int) {
		isQuiting = true
		QuitingFlag = flag
		browserWindow.CloseBrowserWindow()
	}

	//托盘 end
	tray.Show()
}
