package socket

import (
	"encoding/json"
	"fmt"
	"launcher/mproto/entity"

	"google.golang.org/protobuf/proto"
)

// Hub maintains the set of active clients and broadcasts messages to the
// clients.
type Hub struct {
	// Registered clients.
	clients map[*Client]bool

	// Inbound messages from the clients.
	broadcast chan []byte

	// Register requests from the clients.
	register chan *Client

	// Unregister requests from clients.
	unregister chan *Client
}

func newHub() *Hub {
	return &Hub{
		broadcast:  make(chan []byte),
		register:   make(chan *Client),
		unregister: make(chan *Client),
		clients:    make(map[*Client]bool),
	}
}

func BroadMsgObj(msg *entity.NormMsg) {
	if gHub == nil {
		return
	}
	ms, _ := proto.Marshal(msg)
	gHub.broadcast <- ms
}

func BroadMsg(sub string, msg string) {
	if gHub == nil {
		return
	}
	d, _ := json.Marshal(map[string]string{"s": sub, "m": msg})
	gHub.broadcast <- d
}

func (h *Hub) run() {
	for {
		select {
		case client := <-h.register:

			h.clients[client] = true

			fmt.Println("client added")

		case client := <-h.unregister:
			if _, ok := h.clients[client]; ok {
				delete(h.clients, client)
				close(client.send)

				fmt.Println("client remove")
			}
		case message := <-h.broadcast:
			fmt.Println("broadcast msg", len(message))

			for client := range h.clients {
				select {
				case client.send <- message:
				default:
					close(client.send)
					delete(h.clients, client)
				}
			}
		}
	}
}
