package utils

import (
	"bufio"
	"fmt"
	"io"
	"io/ioutil"
	"net"
	"os"
	"os/exec"
	"path"
	"path/filepath"
	"regexp"
	"strings"
	"time"
)

func GetFreePort() (int, error) {
	addr, err := net.ResolveTCPAddr("tcp", "localhost:0")
	if err != nil {
		return 0, err
	}

	l, err := net.ListenTCP("tcp", addr)
	if err != nil {
		return 0, err
	}
	defer l.Close()
	return l.Addr().(*net.TCPAddr).Port, nil
}

func GetAppName() (string, error) {
	p, e := os.Executable()
	if e != nil {
		return "", e
	}
	_, exeName := filepath.Split(p)
	return strings.Split(exeName, ".")[0], nil
}

func GetRealExe(ExeDir string, exeName string) (string, string) {
	version, _ := os.ReadFile(path.Join(ExeDir, "version"))
	if len(version) < 1 {
		version = []byte("0.0.0")
	}
	vd := fmt.Sprintf("v%s", string(version))
	verDir := path.Join(ExeDir, vd, fmt.Sprintf("%s%s.exe", exeName, vd)) //可执行文件
	if !IsExist(verDir) {
		panic("当前版本文件为空")
	}
	return verDir, string(version)
}

// 启动系统bus
func SingleStartApp(ExeDir string, dataDir string, appName string, onStart func(*exec.Cmd)) error {

	realExe, verionStr := GetRealExe(ExeDir, appName)

	isNewStarting := false
	d, _ := os.ReadFile(path.Join(ExeDir, "lastv"))
	lastv := string(d)
	lastDir := path.Join(ExeDir, fmt.Sprintf("v%s", lastv))
	if len(lastv) > 0 { //有lastv文件就是新开
		isNewStarting = true
	}

	buspidfpath := path.Join(dataDir, fmt.Sprintf("%s.exe.pid", appName))
	fmt.Println("app id=>", buspidfpath, IsExist(buspidfpath))
	socketfpath := path.Join(dataDir, fmt.Sprintf("%s.exe.socket", appName))

	if IsExist(buspidfpath) { //启动过

		d, e := ioutil.ReadFile(buspidfpath)
		if e != nil {
			return e
		}

		isRunning, e := isPidRunning(string(d))
		if e != nil {
			return e
		}
		fmt.Println("pre app pid", string(d), " is runing=>", isRunning)

		if isRunning { //正在运行
			if !isNewStarting {
				if onStart != nil {
					onStart(nil)
				}
				return nil
			}

			//todo优雅关闭当前执行的程序而不是kill
			sc, _ := ioutil.ReadFile(socketfpath)
			KillProc2(string(d), string(sc))
		}
		os.Remove(buspidfpath)
		os.Remove(socketfpath)
	}
	fmt.Println("pre app  not runing start new one!!!")
	fmt.Println("is new started=>", isNewStarting)

	// 使用操作系统的 API 重启应用程序
	para := os.Args[1:]
	para = append(para, "--version", verionStr)

	quitsocket, e := GetFreePort()
	if e != nil {
		return e
	}
	para = append(para, "--socket", fmt.Sprintf("%d", quitsocket))

	cmd := exec.Command(realExe, para...)

	//cmd.SysProcAttr = &syscall.SysProcAttr{HideWindow: true, CreationFlags: syscall.CREATE_NEW_PROCESS_GROUP}

	stdoutIn, e := cmd.StdoutPipe()
	if e != nil {
		return e
	}
	reader := bufio.NewReader(stdoutIn)

	chDown := make(chan int, 1)

	go func() {
		regex, _ := regexp.Compile(fmt.Sprintf("%s.exe started", appName))
		checked := false

		for {
			line, err2 := reader.ReadString('\n')
			if err2 != nil || io.EOF == err2 {
				fmt.Println(err2)
				chDown <- 1
				break
			}
			// log.Info(line)
			fmt.Println(line)

			if !checked {
				match := regex.MatchString(line)
				if match {
					checked = true
					chDown <- 2
				}
			}
		}
	}()

	err := cmd.Start()
	if err != nil {
		fmt.Println("starting ", appName, "  error=>", err)
		return err
	}
	pidnum := cmd.Process.Pid
	cmd.Process.Release()

	if isNewStarting {
		if lastv != "0.0.0" {
			os.RemoveAll(lastDir)
		}
		os.Remove(path.Join(ExeDir, "lastv"))
	}

	tag1 := <-chDown
	if tag1 == 1 { //退出
		return fmt.Errorf("启动异常")
	}

	err = ioutil.WriteFile(buspidfpath, []byte(fmt.Sprintf("%d", pidnum)), os.ModePerm)
	if err != nil {
		return err
	}
	err = ioutil.WriteFile(socketfpath, []byte(fmt.Sprintf("%d", quitsocket)), os.ModePerm)
	if err != nil {
		return err
	}

	if onStart != nil {
		onStart(cmd)
	}

	fmt.Println("app cmd wait begining....")

	//等待退出
	<-chDown

	fmt.Println(cmd.ProcessState)

	lv, _ := os.ReadFile(buspidfpath)
	prev := fmt.Sprintf("%d", pidnum)
	if string(lv) == prev {
		os.Remove(buspidfpath)
		fmt.Println("remove =>", buspidfpath)
	}
	fmt.Println("app cmd wait back=>", e)

	return e
}

func isPidRunning(pid string) (bool, error) {

	// 使用WMIC命令获取指定PID的进程信息
	cmd := exec.Command("WMIC", "PROCESS", "WHERE", fmt.Sprintf("ProcessID=%s", pid), "GET", "Caption")
	out, err := cmd.Output()
	if err != nil {
		fmt.Println(err)
		return false, err
	}
	// 解析WMIC命令输出中的应用程序名称
	fmt.Println("WMIC ==>", string(out))
	name := string(out)
	name = strings.TrimSpace(name)
	if len(name) < 1 {
		return false, nil
	}

	name = strings.Split(name, "\n")[1]
	name = strings.TrimSpace(name)

	// 如果获取到的应用程序名称与指定的进程ID匹配，则该应用程序仍在运行
	if strings.ToLower(name) != "system idle process" {
		fmt.Printf("应用程序 %s 正在运行\n", name)
		return true, nil
	}
	fmt.Println("指定的进程ID未找到")
	return false, nil
}

func NotifyQuit(port int) error {
	conn, err := net.DialTimeout("tcp", fmt.Sprintf("localhost:%d", port), time.Second*2)
	if err != nil {
		fmt.Println("NotifyQuit Dial tcp error", err, port)
		return err
	}
	buf := make([]byte, 128)
	n, err := conn.Read(buf)
	if err != nil {
		fmt.Println("Error reading message:", err)
		return err
	}
	fmt.Println("notify quit=>", string(buf[:n]))
	time.Sleep(500 * time.Microsecond) //等待程序完全退出

	return nil
}

func NotifyResume(port int) error {
	conn, err := net.DialTimeout("tcp", fmt.Sprintf("localhost:%d", port), time.Second*2)
	if err != nil {
		fmt.Println("NotifyQuit Dial tcp error", err, port)
		return err
	}

	_, err = conn.Write([]byte("resume"))

	return err
}
